/*  kglband.cpp - 
    This file is part of KGL
    Copyright (C) 2000,2001  Antonio Larrosa Jimenez
    KGL' homepage : http://devel-home.kde.org/~larrosa/kgl.html

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

    Send comments and bug fixes to Antonio Larrosa <larrosa@kde.org>

 ***************************************************************************/
#include "kglband.h"
#include <math.h>
#include <stdio.h>

KGLBand::KGLBand(GLfloat **a, GLfloat **b, int n,
	GLfloat **normala, GLfloat **normalb,QImage texture) : KGLObject(0,0,0)
{
  m_list = glGenLists(1);
  glNewList(m_list, GL_COMPILE);
  GLUquadricObj* q = gluNewQuadric();
  gluQuadricDrawStyle( q, (GLenum) GLU_FILL );
  gluQuadricNormals( q, (GLenum) GLU_SMOOTH ); 
  gluQuadricOrientation( q, (GLenum) GLU_INSIDE );

  if (!texture.isNull())
  {
    gluQuadricTexture(q, GL_TRUE);
    m_texture = texture.convertDepth( 32 ).mirror().swapRGB();
    glTexImage2D( GL_TEXTURE_2D, 0, 4, m_texture.width(), m_texture.height(), 0,
	GL_RGBA, GL_UNSIGNED_BYTE, m_texture.bits() );
  }  
  glBegin(GL_TRIANGLE_STRIP);
  glPolygonMode (GL_FRONT_AND_BACK, GL_FILL);

  for (int i=0;i<n;i++)
  {
    glTexCoord2f(((float)i)/n, 0.1);
    if (normalb) 
      glNormal3fv(normalb[i]);
    //    printf("%f %f %f\n", normalb[i][0], normalb[i][1], normalb[i][2]);
    glVertex3f(b[i][0],b[i][1],b[i][2]);
    glTexCoord2f(((float)i)/n, 0.9);
    if (normala) 
      glNormal3fv(normala[i]);
    //    printf("%f %f %f\n",normala[i][0], normala[i][1], normala[i][2]);
    glVertex3f(a[i][0],a[i][1],a[i][2]);
  }

  glEnd(); 
  gluDeleteQuadric(q);
  glEndList(); 
}

KGLBand::KGLBand(GLfloat **a, GLfloat dx, GLfloat dy, GLfloat dz,
    int n, QImage texture) : KGLObject(0,0,0)
{
  m_list = glGenLists(1);
  glNewList(m_list, GL_COMPILE);
  GLUquadricObj* q = gluNewQuadric();
  gluQuadricDrawStyle( q, (GLenum) GLU_FILL );
  gluQuadricNormals( q, (GLenum) GLU_SMOOTH ); 

  glBegin(GL_TRIANGLE_STRIP);
  if (!texture.isNull())
  {
    gluQuadricTexture(q, GL_TRUE);
    m_texture = texture.convertDepth( 32 ).mirror().swapRGB();
    glTexImage2D( GL_TEXTURE_2D, 0, 4, m_texture.width(), m_texture.height(), 0,
	GL_RGBA, GL_UNSIGNED_BYTE, m_texture.bits() );
  }  

  for (int i=0;i<n;i++)
  {
    if (i<n-1) 
    {
       float x=a[i+1][0]-a[i][0];
       float y=a[i+1][1]-a[i][1];
       float z=a[i+1][2]-a[i][2];
       float vx=dy*z-dz*y;
       float vy=x*dz-dx*z;
       float vz=dx*y-x*dy;
       float n=sqrt( vx*vx+vy*vy+vz*vz );
       glNormal3f( vx*0.8/n, vy*0.8/n, vz*0.8/n );
//       printf("1. %f %f %f\n", dy*z-dz*y,x*dz-dx*z,dx*y-x*dy);
//       printf("2. %f %f %f\n", vx, vy, vz);
    }
    glVertex3f( a[i][0], a[i][1], a[i][2] );
    glVertex3f( a[i][0]+dx ,a[i][1]+dy, a[i][2]+dz );
  }
  for (int i=n-2;i>0;i--)
  {
    if (i<n-1) 
    {
       float x=a[i+1][0]-a[i][0];
       float y=a[i+1][1]-a[i][1];
       float z=a[i+1][2]-a[i][2];
       float vx=-(dy*z-dz*y);
       float vy=-(x*dz-dx*z);
       float vz=-(dx*y-x*dy);
       float n=sqrt( vx*vx+vy*vy+vz*vz );
       glNormal3f( vx*0.8/n, vy*0.8/n, vz*0.8/n );
    }
    glVertex3f( a[i][0], a[i][1], a[i][2] );
    glVertex3f( a[i][0]+dx, a[i][1]+dy, a[i][2]+dz );
  }
  glVertex3f( a[0][0], a[0][1], a[0][2] );
  glVertex3f( a[0][0]+dx, a[0][1]+dy, a[0][2]+dz );

  glEnd(); 
  gluDeleteQuadric(q);
  glEndList(); 
}
