/*  kglelements.h -
    This file is part of KGL
    Copyright (C) 2000,2001  Antonio Larrosa Jimenez
    KGL' homepage : http://devel-home.kde.org/~larrosa/kgl.html
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
    Send comments and bug fixes to Antonio Larrosa <larrosa@kde.org>
 
 ***************************************************************************/
#ifndef _ELEMENTS_H
#define _ELEMENTS_H

#include <qtextstream.h>
#include <math.h>
#include <stdio.h>

class Vector
{
public:
   Vector(float x, float y, float z) { m_x=x; m_y=y; m_z=z; };
   Vector() { m_x=0; m_y=0; m_z=0; };
   float x() const { return m_x; };
   float y() const { return m_y; };
   float z() const { return m_z; };
   float norm() const { return sqrt(m_x*m_x+m_y*m_y+m_z*m_z); };
   Vector &setNorm(float n);
   void toUnit() { setNorm(1.0); };
  
   void print(const char *);
   void toSquare();

   /**
    * Gives the decomposition of this vector with respecto to e 
    */
   Vector decomp(const Vector &e) const;

   /**
    * Returns the projection of this vector to the plane defined by its
    * normal vector, n.
    */
   Vector projectToPlane(const Vector &n) const;

   float setX(float x) { return m_x=x; };
   float setY(float y) { return m_y=y; };
   float setZ(float z) { return m_z=z; };

   Vector &operator+=(const Vector &v)
   { m_x+=v.m_x; m_y+=v.m_y; m_z+=v.m_z; return *this; };
   Vector &operator-=(const Vector &v)
   { m_x-=v.m_x; m_y-=v.m_y; m_z-=v.m_z; return *this; };
   Vector &operator*=(float r)
   {
      m_x*=r; m_y*=r; m_z*=r;
      return *this;
   };

   Vector operator*(float r)
   {
      Vector tmp(*this);
      tmp.m_x*=r; tmp.m_y*=r; tmp.m_z*=r;
      return tmp;
   };
   
protected: 
   float m_x,m_y,m_z;
};

inline float escalarProduct(const Vector &a, const Vector &b)
{
  return a.x()*b.x()+a.y()*b.y()+a.z()*b.z();
}

inline Vector operator+(const Vector &v1, const Vector &v2)
{
  Vector tmp(v1);
  tmp+=v2;
  return tmp;
}

inline Vector operator-(const Vector &v1, const Vector &v2)
{
  Vector tmp(v1);
  tmp-=v2;
  return tmp;
}

inline Vector operator*(const Vector &v1, const Vector &v2)
{
  return Vector( v1.y()*v2.z()-v2.y()*v1.z(),
      -(v1.x()*v2.z()-v2.x()*v1.z()),
      v1.x()*v2.y()-v2.x()*v1.y());
}

class State
{
public:
   Vector m_pos;
   Vector m_vel;
};

class Segment
{

public:
   Vector pt1() const { return m_pt1; };
   Vector pt2() const { return m_pt2; };

protected:
   Vector m_pt1;
   Vector m_pt2;

};

QTextStream &operator<<( QTextStream &s, const Vector &c );
QTextStream &operator>>( QTextStream &s, Vector &c );

#endif
