/*  kglmesh_mesh.cpp - Another way to make meshes. Not used in this moment
    This file is part of KGL
    Copyright (C) 2000  Antonio Larrosa Jimenez
    KGL' homepage : http://devel-home.kde.org/~larrosa/kgl.html

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

    Send comments and bug fixes to Antonio Larrosa <larrosa@kde.org>

 ***************************************************************************/
#include "kglmesh.h"
#include <qgl.h>

KGLMesh::KGLMesh(GLfloat **mesh, GLfloat ***normalMesh, GLfloat _x1,
	GLfloat _y1, GLfloat _x2, GLfloat _y2, int w, int h, QImage texture)
	: KGLObject(0,0,0)
{
  m_list = glGenLists(1);
  glNewList(m_list, GL_COMPILE);
  GLUquadricObj* q = gluNewQuadric();
  gluQuadricDrawStyle( q, (GLenum) GLU_FILL );
  gluQuadricNormals( q, (GLenum) GLU_SMOOTH ); 
 
  if (!texture.isNull())
  {
    gluQuadricTexture(q, GL_TRUE);
    m_texture = texture.convertDepth( 32 ).mirror().swapRGB();
    glTexImage2D( GL_TEXTURE_2D, 0, 4, m_texture.width(), m_texture.height(), 0,
        GL_RGBA, GL_UNSIGNED_BYTE, m_texture.bits() );
  } 

  glBegin(GL_TRIANGLE_STRIP); 
//  glEnable(GL_NORMALIZE);
  glMaterialfv( GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, color() );
//  float front_shininess[] =   {60.0};
//  float front_specular[]  =   { 0.7, 0.7, 0.7, 1.0 };                      
//  glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, front_shininess);
//  glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, front_specular);
  glLightModelf( GL_LIGHT_MODEL_TWO_SIDE, (GLenum) GL_TRUE );


  gluQuadricOrientation( q, (GLenum) GLU_INSIDE );

  float stepx=(_x2-_x1)/(w-1);
  float stepy=(_y2-_y1)/(h-1);

  for (int j=0;j<h-1;j++)
  {
    float x1=_x1;
    float y1=_y1+j*stepy;
    float x2=x1+stepx;
    float y2=y1+stepy;

    GLfloat *a=mesh[j];
    GLfloat *b=mesh[j+1];
    GLfloat **na=normalMesh[j];
    GLfloat **nb=normalMesh[j+1];

    glNormal3fv(na[0]);
    glVertex3f(x1,y1,a[0]);
    glNormal3fv(nb[0]);
    glVertex3f(x1,y2,b[0]);

    for (int i=1;i<w;i++)
    {
      x1=_x1+i*stepx;
      glNormal3fv(na[i]);
      glVertex3f(x1,y1,a[i]);
      glNormal3fv(nb[i]);
      glVertex3f(x1,y2,b[i]);
    }

/*    if (j+2<h)
    {
      j++;

      a=b;
      b=mesh[j+1];
      na=nb;
      nb=normalMesh[j+1];

      y1=y2;
      y2=y1+stepy;
      x1=_x1+(w-1)*stepx;

      for (int i=w-1;i>0;i--)
      {
	x2=x1;
	x1=_x1+(i-1)*stepx;
	glNormal3fv(nb[i]);
	glVertex3f(x2,y2,b[i]);
	glNormal3fv(na[i]);
	glVertex3f(x1,y1,a[i]);
      }
    }
*/
  }
//  glVertex3f(_x1,_y1,mesh[0][0]);
  /*
  gluQuadricOrientation( q, (GLenum) GLU_OUTSIDE );
  a=mesh[0];
  b=mesh[0];
  
  stepx=(_x2-_x1)/(w-1);
  stepy=(_y2-_y1)/(h-1);


  for (int j=0;j<h-1;j++)
  {
    float x1=_x1;
    float y1=_y1+j*stepy;
    float x2=x1+stepx;
    float y2=y1+stepy;

    GLfloat *a=mesh[j];
    GLfloat *b=mesh[j+1];

    glVertex3f(x1,y1,a[0]);
    glVertex3f(x1,y2,b[0]);

    for (int i=1;i<w;i++)
    {
      x1=_x1+i*stepx;
      glVertex3f(x1,y1,a[i]);
      glVertex3f(x1,y2,b[i]);
    }

    if (j+2<h)
    {
      j++;

      a=b;
      b=mesh[j+1];

      y1=y2;
      y2=y1+stepy;
      x1=_x1+(w-1)*stepx;

      for (int i=w-1;i>0;i--)
      {
	x2=x1;
	x1=_x1+(i-1)*stepx;
	glVertex3f(x2,y2,b[i]);
	glVertex3f(x1,y1,a[i]);
      }
    }

  }
*/
  gluQuadricTexture(q, GL_FALSE);
  glEnd(); 
  gluDeleteQuadric(q);
  glEndList(); 
  
}
