/*  kglrectangle.cpp - 
    This file is part of KGL
    Copyright (C) 2000,2001  Antonio Larrosa Jimenez
    KGL' homepage : http://devel-home.kde.org/~larrosa/kgl.html

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

    Send comments and bug fixes to Antonio Larrosa <larrosa@kde.org>

 ***************************************************************************/
#include "kglrectangle.h"
#include "kglelements.h"

KGLRectangle::KGLRectangle(GLfloat x1, GLfloat y1, GLfloat z1,
    GLfloat x2, GLfloat y2, GLfloat z2,GLfloat x3, GLfloat y3, GLfloat z3,
    GLfloat x4, GLfloat y4, GLfloat z4) : KGLObject(0,0,0)
{
  m_list = glGenLists(1);
  glNewList(m_list, GL_COMPILE);
  GLUquadricObj* q = gluNewQuadric();
  gluQuadricDrawStyle( q, (GLenum) GLU_FILL );
  gluQuadricNormals( q, (GLenum) GLU_SMOOTH ); 
 
  glBegin(GL_QUADS); 
  glMaterialfv( GL_FRONT, GL_AMBIENT_AND_DIFFUSE, color() );
  glVertex3f(x1,y1,z1);
  glVertex3f(x2,y2,z2);
  glVertex3f(x3,y3,z3);
  glVertex3f(x4,y4,z4);

  glEnd(); 
  gluDeleteQuadric(q);
  glEndList(); 
  
}

KGLRectangle::KGLRectangle(GLfloat x1, GLfloat y1, GLfloat z1,
    GLfloat x2, GLfloat y2, GLfloat z2, GLfloat x3, GLfloat y3, GLfloat z3)
    : KGLObject(0,0,0)
{
  m_list = glGenLists(1);
  glNewList(m_list, GL_COMPILE);
  GLUquadricObj* q = gluNewQuadric();
  gluQuadricDrawStyle( q, (GLenum) GLU_FILL );
  gluQuadricNormals( q, (GLenum) GLU_SMOOTH ); 
 
  glBegin(GL_QUADS); 
  glMaterialfv( GL_FRONT, GL_AMBIENT_AND_DIFFUSE, color() );
  glVertex3f(x1,y1,z1);
  glVertex3f(x2,y2,z2);
  glVertex3f(x3,y3,z3);
  glVertex3f(x1+(x3-x2),y1+(y3-y2),z1+(z3-z2));

  glEnd(); 
  gluDeleteQuadric(q);
  glEndList(); 
  
} 

KGLRectangle::KGLRectangle(GLfloat x, GLfloat y, GLfloat z,
	GLfloat nx, GLfloat ny, GLfloat nz, GLfloat radius) : KGLObject(0,0,0)
{
  m_list = glGenLists(1);
  glNewList(m_list, GL_COMPILE);
  GLUquadricObj* q = gluNewQuadric();
  gluQuadricDrawStyle( q, (GLenum) GLU_FILL );
  gluQuadricNormals( q, (GLenum) GLU_SMOOTH ); 
 
  glBegin(GL_QUADS); 
  
  Vector pos(x,y,z);
  Vector n(nx,ny,nz);
  Vector e(1,0,0);
  Vector e1=n*e;
  Vector e2=n*e1;
  e1.setNorm(radius);
  e2.setNorm(radius);
  
  Vector v=pos-e1+e2;
  glVertex3f(v.x(),v.y(),v.z());
  v=pos-e1-e2;
  glVertex3f(v.x(),v.y(),v.z());
  v=pos+e1-e2;
  glVertex3f(v.x(),v.y(),v.z());
  v=pos+e1+e2;
  glVertex3f(v.x(),v.y(),v.z());

  glEnd(); 
  gluDeleteQuadric(q);
  glEndList(); 
} 
